/*==================================================================================================
*   Project              : RTD AUTOSAR 4.4
*   Platform             : CORTEXM
*   Peripheral           : IPV_FTFC
*   Dependencies         : 
*
*   Autosar Version      : 4.4.0
*   Autosar Revision     : ASR_REL_4_4_REV_0000
*   Autosar Conf.Variant :
*   SW Version           : 1.0.0
*   Build Version        : S32K1_RTD_1_0_0_HF01_D2109_ASR_REL_4_4_REV_0000_20210907
*
*   (c) Copyright 2020-2021 NXP Semiconductors
*   All Rights Reserved.
*
*   NXP Confidential. This software is owned or controlled by NXP and may only be
*   used strictly in accordance with the applicable license terms. By expressly
*   accepting such terms or by downloading, installing, activating and/or otherwise
*   using the software, you are agreeing that you have read, and that you agree to
*   comply with and are bound by, such license terms. If you do not agree to be
*   bound by the applicable license terms, then you may not retain, install,
*   activate or otherwise use the software.
==================================================================================================*/

#ifndef FTFC_EEP_IP_CFG_H
#define FTFC_EEP_IP_CFG_H

/**
*   @file Ftfc_Eep_Ip_Cfg.h
*
*   @addtogroup FTFC_EEP_IP
*   @{
*/

#ifdef __cplusplus
extern "C"{
#endif

/*==================================================================================================
*                                          INCLUDE FILES
* 1) system and project includes
* 2) needed interfaces from external units
* 3) internal and external interfaces from this unit
==================================================================================================*/

#include "S32K146_FTFC.h"
#include "S32K146_SIM.h"



#include "Ftfc_Eep_Ip_VS_0_PBcfg.h"



#include "Ftfc_Eep_Ip_Types.h"

/*==================================================================================================
*                                 SOURCE FILE VERSION INFORMATION
==================================================================================================*/
#define FTFC_EEP_IP_CFG_VENDOR_ID_H                    43
#define FTFC_EEP_IP_CFG_AR_RELEASE_MAJOR_VERSION_H     4
#define FTFC_EEP_IP_CFG_AR_RELEASE_MINOR_VERSION_H     4
#define FTFC_EEP_IP_CFG_AR_RELEASE_REVISION_VERSION_H  0
#define FTFC_EEP_IP_CFG_SW_MAJOR_VERSION_H             1
#define FTFC_EEP_IP_CFG_SW_MINOR_VERSION_H             0
#define FTFC_EEP_IP_CFG_SW_PATCH_VERSION_H             0

/*==================================================================================================
*                                       FILE VERSION CHECKS
==================================================================================================*/

/* Version checks for Ftfc_Eep_Ip_VS_#_PBcfg.h */
#if (FTFC_EEP_IP_CFG_VENDOR_ID_H != FTFC_EEP_IP_VS_0_PBCFG_VENDOR_ID_H)
    #error "Ftfc_Eep_Ip.h and Ftfc_Eep_Ip_VS_0_PBcfg.h have different vendor IDs!"
#endif
#if ((FTFC_EEP_IP_CFG_AR_RELEASE_MAJOR_VERSION_H    != FTFC_EEP_IP_VS_0_PBCFG_AR_RELEASE_MAJOR_VERSION_H) || \
     (FTFC_EEP_IP_CFG_AR_RELEASE_MINOR_VERSION_H    != FTFC_EEP_IP_VS_0_PBCFG_AR_RELEASE_MINOR_VERSION_H) || \
     (FTFC_EEP_IP_CFG_AR_RELEASE_REVISION_VERSION_H != FTFC_EEP_IP_VS_0_PBCFG_AR_RELEASE_REVISION_VERSION_H) \
    )
    #error "Ftfc_Eep_Ip.h and Ftfc_Eep_Ip_VS_0_PBcfg.h are for different AUTOSAR versions!"
#endif
#if ((FTFC_EEP_IP_CFG_SW_MAJOR_VERSION_H != FTFC_EEP_IP_VS_0_PBCFG_SW_MAJOR_VERSION_H) || \
     (FTFC_EEP_IP_CFG_SW_MINOR_VERSION_H != FTFC_EEP_IP_VS_0_PBCFG_SW_MINOR_VERSION_H) || \
     (FTFC_EEP_IP_CFG_SW_PATCH_VERSION_H != FTFC_EEP_IP_VS_0_PBCFG_SW_PATCH_VERSION_H)    \
    )
    #error "Ftfc_Eep_Ip.h and Ftfc_Eep_Ip_VS_0_PBcfg.h have different SW versions!"
#endif

/* Version checks for Ftfc_Eep_Ip_Types.h */
#if (FTFC_EEP_IP_CFG_VENDOR_ID_H != FTFC_EEP_IP_TYPES_VENDOR_ID_H)
    #error "Ftfc_Eep_Ip_Cfg.h and Ftfc_Eep_Ip_Types.h have different vendor IDs!"
#endif
#if ((FTFC_EEP_IP_CFG_AR_RELEASE_MAJOR_VERSION_H    != FTFC_EEP_IP_TYPES_AR_RELEASE_MAJOR_VERSION_H) || \
     (FTFC_EEP_IP_CFG_AR_RELEASE_MINOR_VERSION_H    != FTFC_EEP_IP_TYPES_AR_RELEASE_MINOR_VERSION_H) || \
     (FTFC_EEP_IP_CFG_AR_RELEASE_REVISION_VERSION_H != FTFC_EEP_IP_TYPES_AR_RELEASE_REVISION_VERSION_H) \
    )
    #error "Ftfc_Eep_Ip_Cfg.h and Ftfc_Eep_Ip_Types.h are for different AUTOSAR versions!"
#endif
#if ((FTFC_EEP_IP_CFG_SW_MAJOR_VERSION_H != FTFC_EEP_IP_TYPES_SW_MAJOR_VERSION_H) || \
     (FTFC_EEP_IP_CFG_SW_MINOR_VERSION_H != FTFC_EEP_IP_TYPES_SW_MINOR_VERSION_H) || \
     (FTFC_EEP_IP_CFG_SW_PATCH_VERSION_H != FTFC_EEP_IP_TYPES_SW_PATCH_VERSION_H)    \
    )
    #error "Ftfc_Eep_Ip_Cfg.h and Ftfc_Eep_Ip_Types.h have different SW versions!"
#endif

/*==================================================================================================
*                                       DEFINES AND MACROS
==================================================================================================*/


/** FlexRAM base address */
#define FTFC_EEP_IP_FLEXRAM_ADDRESS         0x14000000U

/** Total size of EEP */
#define FTFC_EEP_IP_EEP_SIZE                4096U

/** If this is a HT chip derivative the FTFM registers will be used - instead of FTFC. */
#define FTFC_EEP_IP_HIGH_TEMP_CHIP          STD_OFF

/** STD_ON: Access code loaded on job start / unloaded on job end or error */
#define FTFC_EEP_IP_LOAD_AC_INTO_RAM        STD_OFF

/** API for comparing a RAM region with the contents of an EEP region */
#define FTFC_EEP_IP_COMPARE_API             STD_ON

/** Enable / disable the QuickWrites mode */
#define FTFC_EEP_IP_QUICK_WRITES_API        STD_ON

/** Check if source is aligned with half-word and word access. */
#define FTFC_EEP_IP_ALIGNED_RAM_ACCESS      STD_OFF

/** OsIf counter type used in timeout detection */
#define FTFC_EEP_IP_TIMEOUT_TYPE            OSIF_COUNTER_DUMMY
#define FTFC_EEP_IP_ABORT_TIMEOUT           268435456U
#define FTFC_EEP_IP_ASYNC_WRITE_TIMEOUT     268435456U
#define FTFC_EEP_IP_SYNC_WRITE_TIMEOUT      268435456U

/** This macro can be used to deactivate the DEV_ASSERTs */
#define FTFC_EEP_IP_DEV_ERROR_DETECT        STD_ON


/*==================================================================================================
*                                              ENUMS
==================================================================================================*/

/*==================================================================================================
*                                            CONSTANTS
==================================================================================================*/

/*==================================================================================================
*                                  STRUCTURES AND OTHER TYPEDEFS
==================================================================================================*/

/*==================================================================================================
*                                  GLOBAL VARIABLE DECLARATIONS
==================================================================================================*/

/*==================================================================================================
*                                       FUNCTION PROTOTYPES
==================================================================================================*/

#ifdef __cplusplus
}
#endif

/** @} */

#endif /* FTFC_EEP_IP_CFG_H */
